#include "ndefmanager.h"
#include <QNdefRecord>
#include <QByteArray>
#include <QList>

#include <QDeclarativeContext>
#include <QCoreApplication>
#include <QDebug>

QTM_USE_NAMESPACE

ndefManager::ndefManager(Controller *c, QObject *parent) :
    QObject(parent)
{
    this->m_textController = c;
    this->m_log = new QFile("log.txt");
    m_log->open(QIODevice::Append);
    messageToWrite = NULL;
}

ndefManager::~ndefManager()
{
    m_log->close();
}


void ndefManager::writeAutostartTag(QString text)
{
    m_log->write("preparing autotag");
    QNdefRecord autostartRecord;
    autostartRecord.setTypeNameFormat(QNdefRecord::ExternalRtd);
    autostartRecord.setType("cdnm.at:nfcautolaunch");
    QByteArray payLoad;
    payLoad.append(text);
    autostartRecord.setPayload(payLoad);

    // Create a new autostart tag message
    messageToWrite = new QNdefMessage(autostartRecord);
}

void ndefManager::startNfcDetection(void)
{
    m_textController->setText("starting detection");
    nearFieldManager = new QNearFieldManager(this);

    nearFieldManager->setTargetAccessModes(QNearFieldManager::NdefWriteTargetAccess | QNearFieldManager::NdefReadTargetAccess);
    nearFieldManager->startTargetDetection();

    //    connect(nearFieldManager, SIGNAL(targetLost(QNearFieldTarget*)),
    //    this, SLOT(targetLost(QNearFieldTarget*)));
    connect(nearFieldManager, SIGNAL(targetDetected(QNearFieldTarget*)),
            this, SLOT(targetDetected(QNearFieldTarget*)));

    nearFieldManager->registerNdefMessageHandler(this, SLOT(targetMessageDetected(QNdefMessage,QNearFieldTarget*)));
}

void ndefManager::targetMessageDetected(const QNdefMessage &message, QNearFieldTarget *target)
{
    m_log->write("autolaunch target detected");
    m_textController->setText(message.first().payload());
    Q_UNUSED(target);
}

void ndefManager::targetDetected(QNearFieldTarget *target)
{
    m_log->write("got one");
    m_textController->setText("target detected");
    // Handle potential errors emitted by the target
    connect(target, SIGNAL(error(QNearFieldTarget::Error,QNearFieldTarget::RequestId)),
            this, SLOT(targetError(QNearFieldTarget::Error,QNearFieldTarget::RequestId)));
    //    connect(target, SIGNAL(requestCompleted (const QNearFieldTarget::RequestId)),
    //            this, SLOT(requestCompleted(QNearFieldTarget::RequestId)));

    if(messageToWrite) {
        writeNdefMessage(target);
    } else {
        if(target->hasNdefMessage()) {
            connect(target, SIGNAL(ndefMessageRead(QNdefMessage)),
                    this, SLOT(ndefMessageRead(QNdefMessage)));
            target->readNdefMessages();
        }
    }
}

void ndefManager::writeNdefMessage(QNearFieldTarget *target)
{
    m_textController->setText("writing message");
    connect(target, SIGNAL(ndefMessagesWritten()),
            this, SLOT(ndefMessageWritten()));
    target->writeNdefMessages(QList<QNdefMessage>() << *messageToWrite);
    messageToWrite = NULL;
}

void ndefManager::ndefMessageRead(const QNdefMessage &message)
{
    m_log->write("gelesen");
    qDebug() << message.first().payload();
    m_textController->setText(message.first().payload());
    emit gotSomething(QString(message.first().payload()));
}

void ndefManager::ndefMessageWritten()
{
    m_log->write("written");
    m_textController->setText("message written");
    qDebug() << "written!";
    emit gotSomething("Written");
}

void ndefManager::targetError(QNearFieldTarget::Error error, const QNearFieldTarget::RequestId &id)
{
    m_textController->setText(QString("error: %1").arg(error));
    qDebug() << error;
    emit gotSomething(QString(error));
}
